/*****************************************************************************\
  models.cpp : Printer model mapping

  Copyright (c) 1996 - 2001, Hewlett-Packard Co.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. Neither the name of Hewlett-Packard nor the names of its
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
  NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PATENT INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
\*****************************************************************************/


#include "models.h"

APDK_BEGIN_NAMESPACE

extern void HP_strcat(char* str1, const char* str2);

// ordered list used by DevIDtoPT
// this and following list must match
PRINTER_TYPE DevIDMap[MAX_ID_STRING]={
eDJ400,     // DJ 400
eDJ540,     // DJ 540
eDJ600,     // DJ 600
eDJ6xxPhoto,    // DJ 610
eDJ630,     // DJ 630
eDJ6xxPhoto,    // DJ 640
eAP2xxx,     // DJ 656 - HP Bolt
eDJ6xxPhoto,    // DJ 690
eDJ6xx,     // DJ 660
eDJ6xx,     // DJ 670
eDJ6xx,     // DJ 680
eDJ6xx,     // DJ 6xx
eDJ8xx,     // DJ 810
eDJ8x5,     // DJ 825
eDJ8xx,     // DJ 830
eDJ8x5,     // DJ 845
eDJ8xx,     // DJ 840
eDJ8xx,     // DJ 880
eDJ8xx,     // DJ 895
eDJ9xx,     // DJ 930
eDJ9xx,     // DJ 94x
eDJ9xx,     // DJ 950
eDJ9xxVIP,  // DJ 960
eDJ9xx,     // DJ 970
eDJ9xxVIP,  // DJ 980
eDJ9xxVIP,  // DJ 990
eDJ6xx,     // e-printer
eAP2xxx,    // Apollo P-22
eAP21xx,    // Apollo P2000U
eAP2560,    // Apollo P2500/2600
eDJ350,     // DJ 350
eDJ9xx,     // PS P1000
eDJ9xx,     // PS P1100
eDJ9xxVIP,  // PS 1215
eDJ9xxVIP,  // PS 1218
eDJ9xxVIP,  // PS 1115
eDJ9xxVIP,  // PS 1315
ePSP100,    // PS 100
eDJ8xx,     // DJ 1125 - Young Einstein
eDJ9xx,     // DJ 1220 - Escher
eDJ9xx,     // DJ 920 - Thriftway Lite
eDJ9xxVIP,  // DJ 995 - Jeeni
eDJ9xxVIP,  // CP 1160 - Jupiter Pro
eDJ9xxVIP,  // CP 1700 - Euler
};

char *ModelString[MAX_ID_STRING] = {
"HP DeskJet 4",
"DESKJET 540",
"DESKJET 600",
"DESKJET 61",
"DESKJET 63",
"DESKJET 64",
"DESKJET 65",
"DESKJET 69",
"DESKJET 66",
"DESKJET 67",
"DESKJET 68",
"DESKJET 6",
"DESKJET 81",
"DESKJET 825",
"DESKJET 83",
"DESKJET 845",
"DESKJET 84",
"DESKJET 88",
"DESKJET 895",
"DESKJET 93",
"DESKJET 94",
"DESKJET 95",
"DESKJET 96",
"DESKJET 97",
"DESKJET 98",
"DESKJET 99",
"e-printer e20",
"APOLLO P-22",
"P-2000U",  // yes, the P-2100 report itself as a P-2000
"APOLLO P2500/2600",
"DESKJET 350",
"PHOTOSMART P1000",
"PHOTOSMART P1100",
"PHOTOSMART 1215",
"PHOTOSMART 1218",
"PHOTOSMART 1115",
"PHOTOSMART 1315",
"PHOTOSMART 100",
"DESKJET 1125",
"DESKJET 1220",
"DESKJET 92",
"DESKJET 995",
"cp1160",
"cp1700",
};

PRINTER_TYPE BuiltIn[]={    // must match PRINTER_TYPE enum order
#ifdef APDK_DJ400
    eDJ400,
#endif
#ifdef APDK_DJ540
    eDJ540,
#endif
#ifdef APDK_DJ600
    eDJ600,
#endif
#ifdef APDK_DJ6xx
    eDJ6xx,
#endif
#ifdef APDK_DJ6xxPhoto
    eDJ6xxPhoto,
#endif
#ifdef APDK_DJ8xx
    eDJ8xx,
#endif
#ifdef APDK_DJ9xx
    eDJ9xx,
#endif
#ifdef APDK_DJ9xxVIP
    eDJ9xxVIP,
#endif
#ifdef APDK_DJ630
    eDJ630,
#endif
#ifdef APDK_APOLLO2XXX
    eAP2xxx,
#endif
#ifdef APDK_APOLLO21XX
    eAP21xx,
#endif
#ifdef APDK_APOLLO2560
    eAP2560,
#endif
#ifdef APDK_DJ350
    eDJ350,
#endif
#ifdef APDK_DJ8x5
    eDJ8x5,
#endif
#ifdef APDK_PSP100
    ePSP100,
#endif
#ifdef APDK_DJ9xxVIP
    eDJGenericVIP,
#endif
};

unsigned int BuiltInSize =

#ifdef APDK_DJ400
    1 +
#endif
#ifdef APDK_DJ540
    1 +
#endif
#ifdef APDK_DJ600
    1 +
#endif
#ifdef APDK_DJ6xx
    1 +
#endif
#ifdef APDK_DJ6xxPhoto
    1 +
#endif
#ifdef APDK_DJ8xx
    1 +
#endif
#ifdef APDK_DJ9xx
    1 +
#endif
#ifdef APDK_DJ9xxVIP
    2 +
#endif

#ifdef APDK_DJ8x5
    1 +
#endif
#ifdef APDK_DJ630
    1 +
#endif
#ifdef APDK_APOLLO2XXX
    1 +
#endif
#ifdef APDK_APOLLO21XX
    1 +
#endif
#ifdef APDK_APOLLO2560
    1 +
#endif
#ifdef APDK_DJ350
    1 +
#endif
#ifdef APDK_PSP100
    1 +
#endif
    0
    ;

unsigned int modelbits()
{
    unsigned int bits=0;

#ifdef APDK_DJ400
        bits = bits | 0x800000;
#endif
#ifdef APDK_DJ540
        bits = bits | 0x400000;
#endif
#ifdef APDK_DJ600
        bits = bits | 0x200000;
#endif
#ifdef APDK_DJ6xx
        bits = bits | 0x100000;
#endif
#ifdef APDK_DJ6xxPhoto
        bits = bits | 0x80000;
#endif
#ifdef APDK_DJ8xx
        bits = bits | 0x40000;
#endif
#ifdef  APDK_DJ9xx
        bits = bits | 0x20000;
#endif
#ifdef APDK_DJ9xxVIP
        bits = bits | 0x10000;
#endif
#ifdef APDK_DJ630
        bits = bits | 0x8000;
#endif
#ifdef APDK_APOLLO2XXX
        bits = bits | 0x4000;
#endif
#ifdef APDK_APOLLO21XX
        bits = bits | 0x2000;
#endif
#ifdef APDK_APOLLO2560
        bits = bits | 0x1000;
#endif
#ifdef APDK_DJ350
        bits = bits | 0x800;
#endif
#ifdef APDK_DJ8x5
        bits = bits | 0x400;
#endif
#ifdef APDK_PSP100
        bits = bits | 0x200;
#endif

#ifdef APDK_DJ9xxVIP
        bits = bits | 0x80; // eDJGenericVIP
#endif
        return bits;
}

char mresult[300];
char* modelresult()
{
    mresult[0] = '\0';

#ifdef APDK_DJ400
        HP_strcat(mresult,ModelName[eDJ400]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ540
        HP_strcat(mresult,ModelName[eDJ540]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ600
        HP_strcat(mresult,ModelName[eDJ600]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ6xx
        HP_strcat(mresult,ModelName[eDJ6xx]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ6xxPhoto
        HP_strcat(mresult,ModelName[eDJ6xxPhoto]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ8xx
        HP_strcat(mresult,ModelName[eDJ8xx]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ8x5
        HP_strcat(mresult,ModelName[eDJ8x5]); HP_strcat(mresult," ");
#endif
#ifdef  APDK_DJ9xx
        HP_strcat(mresult,ModelName[eDJ9xx]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ9xxVIP
        HP_strcat(mresult,ModelName[eDJ9xxVIP]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ3320
        HP_strcat (mresult, ModelName[eDJ3320]); HP_strcat (mresult, " ");
#endif
#ifdef APDK_DJ630
        HP_strcat(mresult,ModelName[eDJ630]); HP_strcat(mresult," ");
#endif
#ifdef APDK_APOLLO2XXX
        HP_strcat(mresult,ModelName[eAP2xxx]); HP_strcat(mresult," ");
#endif
#ifdef APDK_APOLLO21XX
        HP_strcat(mresult,ModelName[eAP21xx]); HP_strcat(mresult," ");
#endif
#ifdef APDK_APOLLO2560
        HP_strcat(mresult,ModelName[eAP2560]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ350
        HP_strcat(mresult,ModelName[eDJ350]); HP_strcat(mresult," ");
#endif
#ifdef APDK_PSP100
        HP_strcat(mresult,ModelName[ePSP100]); HP_strcat(mresult," ");
#endif
#ifdef APDK_DJ9xxVIP
        HP_strcat(mresult,ModelName[eDJGenericVIP]); HP_strcat(mresult," ");
#endif
    return mresult;
}

APDK_END_NAMESPACE

#ifdef APDK_PROTO
// strings for test harness
#include "..\..\Harness\StdAfx.h"
#include "modes.h"

using namespace APDK_NAMESPACE;

extern CString IDStrings[MAX_PRINTER_TYPE+1][MAX_PRINTMODES];

void InitIDStrings()
{
    // init array with sentinel values -- checked by ProtoServices constructor
    for (int i=0; i<MAX_PRINTER_TYPE; i++)
        for (int j=0; j<MAX_PRINTMODES; j++)
            IDStrings[i][j] = "DUMMY";


    //dj350 w/ both pens
    IDStrings[eDJ350][DEFAULTMODE_INDEX] =
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 350;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 350;VSTATUS:$RC0,ff,IDLE;";
    IDStrings[eDJ350][GRAYMODE_INDEX]    =
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 350;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 350;VSTATUS:$CB0,ff,IDLE;";
    IDStrings[eDJ350][SPECIALMODE_INDEX] =
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 350;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 350;VSTATUS:$CB0,ff,IDLE;";
    IDStrings[eDJ350][EXTRASPECIALMODE_INDEX] =
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 350;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 350;VSTATUS:$RC0,ff,IDLE;";



    IDStrings[eDJ400][GRAYMODE_INDEX]=
        "MFG:Hewlett-Packard;MDL:HP DeskJet 400;CMD:PCL,C32;CLS:PRINTER;DES:HP DeskJet 400 Printer;REV:5.140;DAT:95/07/12;";
    IDStrings[eDJ400][DEFAULTMODE_INDEX]=
        "MFG:Hewlett-Packard;MDL:HP DeskJet 400;CMD:PCL,C32;CLS:PRINTER;DES:HP DeskJet 400 Printer;REV:5.140;DAT:95/07/12;";

    //dj540 w/ color pen
    IDStrings[eDJ540][DEFAULTMODE_INDEX]=
         // temporarily make defaultmode map to black pen
        //"MFG:HEWLETT-PACKARD;MDL:DESKJET 540;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 540;VSTATUS:$DC0,ff,DN,IDLE;";
    //dj540 w/ black pen
    IDStrings[eDJ540][GRAYMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 540;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 540;VSTATUS:$EB0,ff,DN,IDLE;";

    //dj600 w/ color pen
    IDStrings[eDJ600][DEFAULTMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 600;VSTATUS:$RC0,ff,DN,IDLE;";
    IDStrings[eDJ600][GRAYMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 600;VSTATUS:$C0,ff,DN,IDLE;";

    //dj670 w/ both pens
    IDStrings[eDJ6xx][DEFAULTMODE_INDEX]=
    IDStrings[eDJ6xx][GRAYMODE_INDEX]=       // only one possible pen configuration
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 670C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 670C;VSTATUS:$CB0$RC0,ff,DN,IDLE;";

    //dj690 w/ both pens
    IDStrings[eDJ6xxPhoto][DEFAULTMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 690C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 690C;VSTATUS:$CB0$RC0,ff,DN,IDLE;";
    //dj690 w/ photopen
    IDStrings[eDJ6xxPhoto][SPECIALMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 690C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 690C;VSTATUS:$MD0$RC0,ff,DN,IDLE;";
    IDStrings[eDJ6xxPhoto][GRAYMODE_INDEX]=  // compatible with "both"
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 690C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 690C;VSTATUS:$CB0$RC0,ff,DN,IDLE;";

    //dj895c w/both pens (hobbes and flash)
    IDStrings[eDJ8xx][DEFAULTMODE_INDEX]=
    IDStrings[eDJ8xx][GRAYMODE_INDEX]=
    IDStrings[eDJ8xx][SPECIALMODE_INDEX]=
    IDStrings[eDJ8xx][EXTRASPECIALMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 895C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 895C;SERN:US87L2R046GP;VSTATUS:$HB0$FC0,ff,DN,IDLE,CUT;";

//  Phobos
    IDStrings[eDJ8x5][DEFAULTMODE_INDEX]=
    IDStrings[eDJ8x5][GRAYMODE_INDEX]=
    IDStrings[eDJ8x5][SPECIALMODE_INDEX]=
    IDStrings[eDJ8x5][EXTRASPECIALMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 825C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 825C;SERN:US87L2R046GP;VSTATUS:$AU0$FC0,ff,DN,IDLE,CUT;";

    // broadway with phototray
    IDStrings[eDJ9xx][DEFAULTMODE_INDEX]=
    IDStrings[eDJ9xx][SPECIALMODE_INDEX]=
    IDStrings[eDJ9xx][GRAYMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 930C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 895C;SERN:US87L2R046GP;VSTATUS:$HB0$FC0,ff,DN,IDLE,CUT,PH;";

    IDStrings[eDJ630][GRAYMODE_INDEX]=
        // next line is temporary fake out to test composite gray
        // to undo, remove this string and fall through to both-pens string
        //"MFG:HEWLETT-PACKARD;MDL:DESKJET 630C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 630C;VSTATUS:$AU0$RC0,ff,DN,IDLE;";
    //dj630 w/ color pen (R) and Black Pen (C)
    IDStrings[eDJ630][SPECIALMODE_INDEX]="MFG:HEWLETT-PACKARD;MDL:DESKJET 630C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 630C;VSTATUS:$CB0$RC0,ff,DN,IDLE;";
    //dj630 w/ color pen (R) and Photo Pen (M)
    IDStrings[eDJ630][EXTRASPECIALMODE_INDEX]="MFG:HEWLETT-PACKARD;MDL:DESKJET 630C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 630C;VSTATUS:$MD0$RC0,ff,DN,IDLE;";
    //dj630 w/ color pen (R) and No Pen (A)
    IDStrings[eDJ630][DEFAULTMODE_INDEX]="MFG:HEWLETT-PACKARD;MDL:DESKJET 630C;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Hewlett-Packard DeskJet 630C;VSTATUS:$AU0$RC0,ff,DN,IDLE;";

    IDStrings[eDJ9xxVIP][GRAYMODE_INDEX]=
    IDStrings[eDJ9xxVIP][DEFAULTMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 960C;CMD:MLC,PCL,PML;CLS:PRINTER;DES:Hewlett-Packard DeskJet 960C;SN:US04C3J02WLY;S:00808880800010002C1100046C2080026;P:0800,FL,B0;J;";

    IDStrings[eAP2560][GRAYMODE_INDEX]=      // black pen
        "MFG:APOLLO;MDL:APOLLO P2500/2600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2560][DEFAULTMODE_INDEX]=   // color pen only
        "MFG:APOLLO;MDL:APOLLO P2500/2600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2560][SPECIALMODE_INDEX]=   // color and black
        "MFG:APOLLO;MDL:APOLLO P2500/2600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2560][EXTRASPECIALMODE_INDEX]=  // MDL
        "MFG:APOLLO;MDL:APOLLO P2500/2600;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$MD0$RC0,ff,DN,IDLE,CUT,PH;";

    // PhotoSmart P100.
    // Currently, the device supports only one mode, so map both to the same one

    IDStrings[ePSP100][GRAYMODE_INDEX]=
    IDStrings[ePSP100][DEFAULTMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:PHOTOSMART 100;CMD:MLC,PCL,PML;CLS:PRINTER;DES:Hewlett-Packard photosmart 100;SN:US04C3J02WLY;S:00808880800010002C1100046C2080026;P:0800,FL,B0;J;";


    IDStrings[eAP2xxx][GRAYMODE_INDEX]=      // black pen
       // "MFG:APOLLO;MDL:APOLLO P-22;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2xxx][DEFAULTMODE_INDEX]=   // color pen only
       // "MFG:APOLLO;MDL:APOLLO P-22;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2xxx][SPECIALMODE_INDEX]=   // color and black
        "MFG:APOLLO;MDL:APOLLO P-22;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP2xxx][EXTRASPECIALMODE_INDEX]=  // MDL
        "MFG:APOLLO;MDL:APOLLO P-22;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$MD0$RC0,ff,DN,IDLE,CUT,PH;";

    IDStrings[eAP21xx][GRAYMODE_INDEX]=      // black pen
        "MFG:APOLLO;MDL:P-2000U;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP21xx][DEFAULTMODE_INDEX]=   // color pen only
        "MFG:APOLLO;MDL:P-2000U;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP21xx][SPECIALMODE_INDEX]=   // color and black
        "MFG:APOLLO;MDL:P-2000U;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$CB0$RB0,ff,DN,IDLE,CUT,PH;";
    IDStrings[eAP21xx][EXTRASPECIALMODE_INDEX]=  // MDL
        "MFG:APOLLO;MDL:P-2000U;CMD:MLC,PCL,PML;CLASS:PRINTER;DESCRIPTION:Apollo DeskJet 1200C;SERN:US87L2R046GP;VSTATUS:$MD0$RC0,ff,DN,IDLE,CUT,PH;";

//  Generic VIP printers

    IDStrings[eDJGenericVIP][GRAYMODE_INDEX]=
    IDStrings[eDJGenericVIP][DEFAULTMODE_INDEX]=
        "MFG:HEWLETT-PACKARD;MDL:DESKJET 990C;CMD:MLC,PCL,PML;CLS:PRINTER;DES:Hewlett-Packard DeskJet 990C;SN:US04C3J02WLY;S:00808880800010002C1100046C2080026;P:0800,FL,B0;J;";

}
#endif //APDK_PROTO


